""" UI Mesh python modules.
The MainWindow module: tkinter_main_app.py
The FirstWindow: upload_magnetogram.py

To start the Main UI:

from mesh_generator.ui.upload_magnetogram import MeshGeneratorUI
>>> MeshGeneratorUI()
"""

from mesh_generator.ui.upload_magnetogram import MeshGeneratorUI
